@@ -7,7 +7,7 @@ use POSIX qw(:fcntl_h);
 use strict;
 use warnings;
 
-our $VERSION = 0.01;
+our $VERSION = 0.02;
 
 sub new {
     my $klass = shift;
@@ -42,6 +42,11 @@ sub DESTROY {
 sub update {
     my ($self, $status) = @_;
     # open file at the first invocation (tmpfn => lock => rename)
+    if ($self->{fh} && $self->{pid_for_fh} != $$) {
+        # fork? close but do not unlock
+        close $self->{fh};
+        undef $self->{fh};
+    }
     unless ($self->{fh}) {
         my $fn = $self->_build_filename();
         open my $fh, '>', "$fn.tmp"
@@ -52,6 +57,7 @@ sub update {
         rename "$fn.tmp", $fn
             or die "failed to rename file:$fn.tmp to $fn:$!";
         $self->{fh} = $fh;
+        $self->{pid_for_fh} = $fh;
     }
     # write to file with size of the status and its checksum
     seek $self->{fh}, SEEK_SET, 0
@@ -108,7 +114,7 @@ sub _for_all {
             or next;
         my $pid = $1;
         # ignore files removed after glob but before open
-        open my $fh, '<', $fn
+        open my $fh, '+<', $fn
             or next;
         # check if the file is still opened by the owner process using flock
         if ($pid != $$ && flock $fh, LOCK_EX | LOCK_NB) {
